#!/usr/bin/env python3
#
# Copyright (C) 2022 Leandro Lisboa Penz <lpenz@lpenz.org>
# This file is subject to the terms and conditions defined in
# file 'LICENSE', which is part of this source code package.

import setuptools

if __name__ == "__main__":
    setuptools.setup()
